@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.008003;
 
-our $VERSION = '1.644';
+our $VERSION = '1.645';
 
 use Exporter qw( import );
 
@@ -322,9 +322,7 @@ sub table_char_width { [
 [   0xe4f,    0xeb0, 1],
 [   0xeb1,    0xeb1, 0],
 [   0xeb2,    0xeb3, 1],
-[   0xeb4,    0xeb9, 0],
-[   0xeba,    0xeba, 1],
-[   0xebb,    0xebc, 0],
+[   0xeb4,    0xebc, 0],
 [   0xebd,    0xec7, 1],
 [   0xec8,    0xecd, 0],
 [   0xece,    0xf17, 1],
@@ -803,8 +801,8 @@ sub table_char_width { [
 [  0xa9b4,   0xa9b5, 1],
 [  0xa9b6,   0xa9b9, 0],
 [  0xa9ba,   0xa9bb, 1],
-[  0xa9bc,   0xa9bc, 0],
-[  0xa9bd,   0xa9e4, 1],
+[  0xa9bc,   0xa9bd, 0],
+[  0xa9be,   0xa9e4, 1],
 [  0xa9e5,   0xa9e5, 0],
 [  0xa9e6,   0xaa28, 1],
 [  0xaa29,   0xaa2e, 0],
@@ -984,7 +982,13 @@ sub table_char_width { [
 [ 0x1182f,  0x11837, 0],
 [ 0x11838,  0x11838, 1],
 [ 0x11839,  0x1183a, 0],
-[ 0x1183b,  0x11a00, 1],
+[ 0x1183b,  0x119d3, 1],
+[ 0x119d4,  0x119d7, 0],
+[ 0x119d8,  0x119d9, 1],
+[ 0x119da,  0x119db, 0],
+[ 0x119dc,  0x119df, 1],
+[ 0x119e0,  0x119e0, 0],
+[ 0x119e1,  0x11a00, 1],
 [ 0x11a01,  0x11a0a, 0],
 [ 0x11a0b,  0x11a32, 1],
 [ 0x11a33,  0x11a38, 0],
@@ -1032,21 +1036,29 @@ sub table_char_width { [
 [ 0x11d97,  0x11d97, 0],
 [ 0x11d98,  0x11ef2, 1],
 [ 0x11ef3,  0x11ef4, 0],
-[ 0x11ef5,  0x16aef, 1],
+[ 0x11ef5,  0x1342f, 1],
+[ 0x13430,  0x13438, 0],
+[ 0x13439,  0x16aef, 1],
 [ 0x16af0,  0x16af4, 0],
 [ 0x16af5,  0x16b2f, 1],
 [ 0x16b30,  0x16b36, 0],
-[ 0x16b37,  0x16f8e, 1],
+[ 0x16b37,  0x16f4e, 1],
+[ 0x16f4f,  0x16f4f, 0],
+[ 0x16f50,  0x16f8e, 1],
 [ 0x16f8f,  0x16f92, 0],
 [ 0x16f93,  0x16fdf, 1],
-[ 0x16fe0,  0x16fe1, 2],
-[ 0x16fe2,  0x16fff, 1],
-[ 0x17000,  0x187f1, 2],
-[ 0x187f2,  0x187ff, 1],
+[ 0x16fe0,  0x16fe3, 2],
+[ 0x16fe4,  0x16fff, 1],
+[ 0x17000,  0x187f7, 2],
+[ 0x187f8,  0x187ff, 1],
 [ 0x18800,  0x18af2, 2],
 [ 0x18af3,  0x1afff, 1],
 [ 0x1b000,  0x1b11e, 2],
-[ 0x1b11f,  0x1b16f, 1],
+[ 0x1b11f,  0x1b14f, 1],
+[ 0x1b150,  0x1b152, 2],
+[ 0x1b153,  0x1b163, 1],
+[ 0x1b164,  0x1b167, 2],
+[ 0x1b168,  0x1b16f, 1],
 [ 0x1b170,  0x1b2fb, 2],
 [ 0x1b2fc,  0x1bc9c, 1],
 [ 0x1bc9d,  0x1bc9e, 0],
@@ -1084,7 +1096,11 @@ sub table_char_width { [
 [ 0x1e023,  0x1e024, 0],
 [ 0x1e025,  0x1e025, 1],
 [ 0x1e026,  0x1e02a, 0],
-[ 0x1e02b,  0x1e8cf, 1],
+[ 0x1e02b,  0x1e12f, 1],
+[ 0x1e130,  0x1e136, 0],
+[ 0x1e137,  0x1e2eb, 1],
+[ 0x1e2ec,  0x1e2ef, 0],
+[ 0x1e2f0,  0x1e8cf, 1],
 [ 0x1e8d0,  0x1e8d6, 0],
 [ 0x1e8d7,  0x1e943, 1],
 [ 0x1e944,  0x1e94a, 0],
@@ -1152,27 +1168,35 @@ sub table_char_width { [
 [ 0x1f6cc,  0x1f6cc, 2],
 [ 0x1f6cd,  0x1f6cf, 1],
 [ 0x1f6d0,  0x1f6d2, 2],
-[ 0x1f6d3,  0x1f6ea, 1],
+[ 0x1f6d3,  0x1f6d4, 1],
+[ 0x1f6d5,  0x1f6d5, 2],
+[ 0x1f6d6,  0x1f6ea, 1],
 [ 0x1f6eb,  0x1f6ec, 2],
 [ 0x1f6ed,  0x1f6f3, 1],
-[ 0x1f6f4,  0x1f6f9, 2],
-[ 0x1f6fa,  0x1f90f, 1],
-[ 0x1f910,  0x1f93e, 2],
-[ 0x1f93f,  0x1f93f, 1],
-[ 0x1f940,  0x1f970, 2],
-[ 0x1f971,  0x1f972, 1],
+[ 0x1f6f4,  0x1f6fa, 2],
+[ 0x1f6fb,  0x1f7df, 1],
+[ 0x1f7e0,  0x1f7eb, 2],
+[ 0x1f7ec,  0x1f90c, 1],
+[ 0x1f90d,  0x1f971, 2],
+[ 0x1f972,  0x1f972, 1],
 [ 0x1f973,  0x1f976, 2],
 [ 0x1f977,  0x1f979, 1],
-[ 0x1f97a,  0x1f97a, 2],
-[ 0x1f97b,  0x1f97b, 1],
-[ 0x1f97c,  0x1f9a2, 2],
-[ 0x1f9a3,  0x1f9af, 1],
-[ 0x1f9b0,  0x1f9b9, 2],
-[ 0x1f9ba,  0x1f9bf, 1],
-[ 0x1f9c0,  0x1f9c2, 2],
-[ 0x1f9c3,  0x1f9cf, 1],
-[ 0x1f9d0,  0x1f9ff, 2],
-[ 0x1fa00,  0x1ffff, 1],
+[ 0x1f97a,  0x1f9a2, 2],
+[ 0x1f9a3,  0x1f9a4, 1],
+[ 0x1f9a5,  0x1f9aa, 2],
+[ 0x1f9ab,  0x1f9ad, 1],
+[ 0x1f9ae,  0x1f9ca, 2],
+[ 0x1f9cb,  0x1f9cc, 1],
+[ 0x1f9cd,  0x1f9ff, 2],
+[ 0x1fa00,  0x1fa6f, 1],
+[ 0x1fa70,  0x1fa73, 2],
+[ 0x1fa74,  0x1fa77, 1],
+[ 0x1fa78,  0x1fa7a, 2],
+[ 0x1fa7b,  0x1fa7f, 1],
+[ 0x1fa80,  0x1fa82, 2],
+[ 0x1fa83,  0x1fa8f, 1],
+[ 0x1fa90,  0x1fa95, 2],
+[ 0x1fa96,  0x1ffff, 1],
 [ 0x20000,  0x2fffd, 2],
 [ 0x2fffe,  0x2ffff, 1],
 [ 0x30000,  0x3fffd, 2],
